// Colour.cpp: Implementation of the Colour class.
//			   This class stores the representation of a colour 
//			   and provides override operators too perform arithmetic
//			   on the colour values.
//////////////////////////////////////////////////////////////////////

#include <assert.h>
#include "Colour.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Colour::Colour()
{
	R = 0.0;
	G = 0.0;
	B = 0.0;
}

Colour::~Colour()
{
}

Colour::Colour(const double r, const double g, const double b)
{
	R = r;
	G = g;
	B = b;
}

void Colour::Set(const double r, const double g, const double b)
{
	R = r;
	G = g;
	B = b;
}

//////////////////////////////////////////////////////////////////////
// Operations
//////////////////////////////////////////////////////////////////////
Colour Colour::Mult(const Colour &colour)
{
    Colour c;
    c.R = R * colour.R;
    c.G = G * colour.G;
    c.B = B * colour.B;
    return c;
}

void Colour::MultScalar(const double a)
{
	R *= a;
	G *= a;
	B *= a;
}

void Colour::Bind()
{
	assert(R>=0 && G>=0 && B>=0 );
	
	if ( R > 1 ) R = 1;
	if ( G > 1 ) G = 1;
    if ( B > 1 ) B = 1;
}

Colour operator * (const double s, const Colour c)
{
	Colour color = c;
	color.MultScalar(s);
	return color;
}

Colour& Colour::operator =(const Colour& c)
{
	R = c.R;
	G = c.G;
	B = c.B;
	return *this;
}

void Colour::operator *=(const double s)
{
	R *= s;
	G *= s;
	B *= s;
}
void Colour::operator /=(const double s)
{
    R /= s;
	G /= s;
	B /= s;
}
void Colour::operator +=(const Colour c)
{
    R += c.R;
	G += c.G;
	B += c.B;
}

Colour Colour::operator * (const Colour& color)
{
    Colour c;
    c.R = R*color.R;
    c.G = G*color.G;
    c.B = B*color.B;
    return c;
}
