// Cylinder.cpp: Implementation of the Cylinder class.
//			Parent Class: Object
//			This class stores the representation of a Cylinder object	
//			and provides a method to intersect a ray with the cylinder
//////////////////////////////////////////////////////////////////////

#include "Cylinder.h"
#include "Misc.h"
#include <iostream.h>


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Cylinder::Cylinder()
{
	this->radius = 1;
	this->height = 1;
}

Cylinder::Cylinder(double rad, double heig)
{
	this->radius = rad;
	this->height = heig;
}

Cylinder::~Cylinder()
{
}

void Cylinder::Set(double rad, double heig)
{
	this->radius = rad;
	this->height = heig;
}

//////////////////////////////////////////////////////////////////////
// Intersect : Tests for intersection between the given ray.  If there
// is an intersection, the hit information is stored in ho, and true
// is returned, otherwise false is returned.
//////////////////////////////////////////////////////////////////////
bool Cylinder::Intersect(Ray ray, HitObject& ho)
{
	double a,b,c,d;
	
	bool cylhit1 = false, cylhit2 = false;
	bool caphit1 = false, caphit2 = false;
	bool cylhit = false, caphit = false;

	double x1,y1,z1,x2,y2,z2,xcyl,ycyl,zcyl,xcap,ycap,zcap;
	double tcap,tcyl,t1,t2;
	double dx,dz;

	Vec3 normal;
	
	Ray newRay = ray.InverseTransRay(this->transMatrix);

	// Get the quadratic for the cylinder body:
	// a t^2 + b t + c = 0

	a = sqr(newRay.v3Dir.x) + sqr(newRay.v3Dir.z);
  
	b = 2*(newRay.v4Ori.x * newRay.v3Dir.x + newRay.v4Ori.z * newRay.v3Dir.z);
  
	c = sqr(newRay.v4Ori.x) + sqr(newRay.v4Ori.z) - sqr(radius);


	// discriminant
	d = b*b -4*a*c;
  
	if (d < 0.0) 
	{
		// no hit
	}
	else 
	{
		t1 = (-b - sqrt(d))/(2*a);
		
		if (t1 > 0) 
		{    
			x1 = newRay.v4Ori.x + t1 * newRay.v3Dir.x;
			y1 = newRay.v4Ori.y + t1 * newRay.v3Dir.y;
			z1 = newRay.v4Ori.z + t1 * newRay.v3Dir.z;

			if (-height/2 < y1 && y1 < height/2) 
				cylhit1 = true;
		}

		t2 = (-b + sqrt(d))/(2*a);
		
		if (t2 > 0) 
		{    
			x2 = newRay.v4Ori.x + t2 * newRay.v3Dir.x;
			y2 = newRay.v4Ori.y + t2 * newRay.v3Dir.y;
			z2 = newRay.v4Ori.z + t2 * newRay.v3Dir.z;

			if (-height/2 < y2 && y2 < height/2) 
				cylhit2 = true;			
		}

		if ((cylhit1 && cylhit2 && t1 < t2) ||
			(cylhit1 && !cylhit2)) 
		{
			xcyl = x1;
			ycyl = y1;
			zcyl = z1;
			tcyl = t1;
			cylhit = true;
		}
		else if ((cylhit1 && cylhit2 && t1 > t2) ||
			(!cylhit1 && cylhit2)) 
		{
			xcyl = x2;
			ycyl = y2;
			zcyl = z2;
			tcyl = t2;
			cylhit = true;
		}
		else 
		{
			cylhit = false;
		}
	}

	//
	// Now, get the hitpoints on the end caps, if any.
	//
	bool caphit2b; 	
	
	t1 = (height/2 - newRay.v4Ori.y) / newRay.v3Dir.y;
	
	if (t1 > 0) 
	{    
		x1 = newRay.v4Ori.x + t1 * newRay.v3Dir.x;
		y1 = newRay.v4Ori.y + t1 * newRay.v3Dir.y;
		z1 = newRay.v4Ori.z + t1 * newRay.v3Dir.z;
    
		dx = x1;
		dz = z1;

		if (dx*dx + dz*dz < radius * radius) 
		  caphit1 = true;
	}
	
	t2 = (-height/2 - newRay.v4Ori.y) / newRay.v3Dir.y;
	
	if (t2 > 0) 
	{    
	    x2 = newRay.v4Ori.x + t2 * newRay.v3Dir.x;
	    y2 = newRay.v4Ori.y + t2 * newRay.v3Dir.y;
	    z2 = newRay.v4Ori.z + t2 * newRay.v3Dir.z;
    
	    dx = x2;
		dz = z2;
		
		if (dx*dx + dz*dz < radius*radius) 
			caphit2 = true;
	}

	if ((caphit1 && caphit2 && t1 < t2) ||
		(caphit1 && !caphit2)) 
	{
		xcap = x1;
		ycap = y1;
		zcap = z1;
		tcap = t1;
		caphit2b = false;
	    caphit = true;
	}
	else if ((caphit1 && caphit2 && t1 > t2) ||
			(!caphit1 && caphit2)) 
	{
		xcap = x2;
		ycap = y2;
		zcap = z2;
		tcap = t2;
		caphit2b = true;
		caphit = true;
	}
	else 
		caphit = false;
  

	// if there were any hits, get the one with 
	// the minimum t value, calculate it's normal,
	// store this info in a hit object and return 
	// true, otherwise return false if there were
	// no hits.
	if ((cylhit && caphit && tcyl < tcap) ||
		(cylhit && !caphit)) 
	{
		// Cylinder hit

		Vec3 hitpoint;
	  
		hitpoint.Set(xcyl, ycyl, zcyl);
	
		normal.Set(hitpoint.x, 0, hitpoint.z);

		ho.Set(ray, tcyl, normal);

		return true;
	}
	else if ((cylhit && caphit && tcyl > tcap) ||
			(!cylhit && caphit)) 
	{
		// Cap Hit

		if (caphit2b == false)
			normal.Set(0, 1,0);
		else
			normal.Set(0,-1,0);

		ho.Set(ray, tcap, normal);

	    return true;
	}
	else
	    return false;
}
