// Scene.h: Interface for the Scene class.
//			This class defines a scene for our ray tracer.  It contains
//			the list of objects in the scene and operations to 
//			get properties of those objects and transform the objects.
//			It also has a method to calculate intersections and shading.
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SCENE_H__B8B3C0CC_A50D_4B65_870A_9DD9937FB8FB__INCLUDED_)
#define AFX_SCENE_H__B8B3C0CC_A50D_4B65_870A_9DD9937FB8FB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Ray.h"
#include "HitObject.h"
#include "SurfaceModel.h"
#include "Light.h"

#define MAX_OBJECT 10

// Constants
#define SPHERE 0
#define PLANE  1
#define POLYGON 2
#define CONE 3
#define CYLINDER 4

// Required classes
class Ray;
class HitObject;
class Color;
class Object;
class Light;

class Scene  
{
public:
	// Definition of a Scene
	int NumObjects;				// Number of Objects
	Object **Item;				// Array of Objects
	int Type[MAX_OBJECT];		// Array to store type of each object
	
	// Constructor & Destructor
	Scene();
	virtual ~Scene();

	// Performs shading of one object
	Colour Shade(const Ray R, const int which, const HitObject ho, Light light) const;

	// checks for intersection with all the objects in the scene and ray.  returns
	// true if at least one hit occurred and which gets the closest object.
	bool Intersects(Ray ray, HitObject& ho, int& which, const int trans ) const;

	// methods to add objects to the scene
	bool AddSphere(double r, SurfaceModel surface);
	bool AddPolygon(int n, Vec4 *P, SurfaceModel surface);
	bool AddCylinder(double r, double height, SurfaceModel surface);
	bool AddCone(double r, double height, SurfaceModel surface);
    	
	// methods to transform the last object added to the scene
	void scale(double sx, double sy, double sz);
	void translate(double tx, double ty, double tz);
	void rotateX(double angle);
	void rotateY(double angle);
	void rotateZ(double angle);

	// returns the transformation matrix of last object added to the scene
	Matrix GetMatrix() const;

	// methods to get the surface properties of the passed in object
    double IsReflective(const int which) const;
    double IsTransparent(const int which) const;
	Colour GetAmbient(const int which) const;
	Colour GetSpecular(const int which) const;
    double GetSpecExponent(const int which) const;
	double GetRefractionIndex(const int which) const;
};

#endif // !defined(AFX_SCENE_H__B8B3C0CC_A50D_4B65_870A_9DD9937FB8FB__INCLUDED_)

