// Sphere.cpp: implementation of the Sphere class.
//			 Parent Class: Object
//			 This class stores the representation of a Sphere object	
//			 and provides a method to intersect a ray with the sphere
//////////////////////////////////////////////////////////////////////

#include "Sphere.h"
#include "Misc.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Sphere::Sphere()
{
	radius = 0.0f;
	center.Set(0.0f,0.0f,0.0f,1.0f);
}

Sphere::~Sphere()
{

}

Sphere::Sphere(const Sphere& s)
{
	center = s.center;
	radius = s.radius;
}

Sphere::Sphere(double x, double y, double z, double rad)
{
	radius = rad;
	center.Set(0, 0, 0, 1);	
}


//////////////////////////////////////////////////////////////////////
// Set Methods
//////////////////////////////////////////////////////////////////////
void Sphere::Set(const Sphere& s)
{
	center = s.center;
	radius = s.radius;	
}

void Sphere::Set(double x, double y, double z, double rad)
{
	radius = rad;
	center.Set(0, 0, 0, 1);	
}

//////////////////////////////////////////////////////////////////////
// Intersect : Tests for intersection between the given ray. If there
// is an intersection, the hit information is stored in ho, and true
// is returned, otherwise false is returned.
//////////////////////////////////////////////////////////////////////
bool Sphere::Intersect(Ray ray, HitObject& ho)
{
	double A, B, C, D, rootD, t1, t2;

	// transform the ray according to this objects transformation
	// matrix
	Ray newRay = ray.InverseTransRay(this->transMatrix);
	

	Vec3 rayOri ( newRay.v4Ori.x,
			      newRay.v4Ori.y,
			      newRay.v4Ori.z );

	Vec3 rayDir;  

	rayDir = newRay.v3Dir;
	
	A = sqr(rayDir);
	B = (rayOri * rayDir);
	C = sqr(rayOri) - sqr(radius);
	D = B * B - A * C;

	// not intersection 
	if( D < 0.0 ) return false;

	rootD = (double)sqrt(D);
	t1 = (-B - rootD )/(A);
	t2 = (-B + rootD )/(A);

	// use 't' in original ray
	// to find actual hit
	if( t1 > EPSILON ) 
	{
		ho.Set(ray, t1, center);
		return true;
	}
	if( t2 > EPSILON )
	{
		ho.Set(ray, t2, center);
		return true;
	}
	
	return false;
}
